package temperature

import (
	"context"
	"fmt"
	"log"
	"os"

	"github.com/go-flucky/flucky/pkg/storage"
	"github.com/go-flucky/flucky/pkg/types"

	"github.com/go-flucky/flucky/pkg/cli"
	"github.com/go-flucky/flucky/pkg/config"
	"github.com/spf13/cobra"
)

var listTemperatureCmd = &cobra.Command{
	Use:     "list",
	Short:   "List temperature values from different or specified sensors by arguments",
	Example: fmt.Sprintf("flucky temperature logs"),
	Run: func(cmd *cobra.Command, args []string) {

		// read configuration
		cnf, err := config.Read(*configFile)
		if err != nil {
			log.Fatalln(err)
		}

		ctx := context.Background()
		storageEndpoint, err := cnf.GetStorageEndpointURL()
		if err != nil {
			log.Fatalln(err)
		}

		measuredValues, err := storage.Read(ctx, storageEndpoint)
		if err != nil {
			log.Fatalln(err)
		}

		measuredValues = types.SelectMeasuredValues(types.MeasuredValueTypeTemperature, measuredValues)

		cli.PrintMeasuredValues(measuredValues, cnf, os.Stdout)

	},
}

func init() {
	temperatureCmd.AddCommand(listTemperatureCmd)
}
