package types

import (
	"encoding/json"
	"fmt"
	"io"
	"time"
)

// Sensor ...
type Sensor struct {
	ID           string      `json:"id" xml:"id"`
	Name         string      `json:"name" xml:"name"`
	Location     string      `json:"location" xml:"location"`
	WireID       *string     `json:"wire_id" xml:"wire_id"`
	I2CBus       *int        `json:"i2c_bus" xml:"i2c_bus"`
	I2CAddress   *uint8      `json:"i2c_address" xml:"i2c_address"`
	GPIONumber   *GPIO       `json:"gpio_number" xml:"gpio_number"`
	Model        SensorModel `json:"model" xml:"model"`
	Enabled      bool        `json:"enabled" xml:"enabled"`
	LastContact  *time.Time  `json:"last_contact" xml:"last_contact"`
	TickDuration string      `json:"tick_duration" xml:"tick_duration"`
	DeviceID     string      `json:"device_id" xml:"device_id"`
	CreationDate time.Time   `json:"creation_date" xml:"creation_date"`
}

// JSONDecoder decodes a json into a sensor
func (s *Sensor) JSONDecoder(r io.Reader) error {
	decoder := json.NewDecoder(r)
	err := decoder.Decode(s)
	if err != nil {
		return fmt.Errorf("Can not decode sensor from json: %v", err)
	}
	return nil
}

func (s *Sensor) JSONEncoder(w io.Writer) error {
	encoder := json.NewEncoder(w)
	encoder.SetIndent("", " ")
	err := encoder.Encode(s)
	if err != nil {
		return fmt.Errorf("Can not encode sensor to json: %v", err)
	}
	return nil
}

func (s *Sensor) FullName() string {
	if s.Name != "" {
		return s.Name
	} else if s.WireID != nil {
		return *s.WireID
	} else if s.I2CAddress != nil &&
		s.I2CBus != nil {
		return fmt.Sprintf("%v/%v", *s.I2CBus, *s.I2CAddress)
	}
	return s.ID
}
